from pynecone import ProtoShell, ProtoCmd


class ItemCmd(ProtoShell):

    class Delete(ProtoCmd):

        def __init__(self):
            super().__init__('delete', 'delete an item')

    class Put(ProtoCmd):

        def __init__(self):
            super().__init__('put', 'put an item')

    class Get(ProtoCmd):

        def __init__(self):
            super().__init__('get', 'get an item')

    class List(ProtoCmd):

        def __init__(self):
            super().__init__('list', 'list items')

    def __init__(self):
        super().__init__(   "item",
                            [ItemCmd.Get(), ItemCmd.Put(), ItemCmd.List(), ItemCmd.Delete()],
                            "realnet items")




from pynecone import Shell, Config

from .itemcmd import ItemCmd
from .typecmd import TypeCmd
from .device import Device

import importlib
import pkgutil
import modules


def iter_namespace(ns_pkg):
    # Specifying the second argument (prefix) to iter_modules makes the
    # returned name an absolute name instead of a relative one. This allows
    # import_module to work without having to do additional modification to
    # the name.
    print(ns_pkg.__path__)
    return pkgutil.iter_modules(ns_pkg.__path__, ns_pkg.__name__ + ".")

modules = [name for _, name, _ in iter_namespace(modules)]

print(modules)

def list_commands():
    entry_mods = [m.split('_')[1] for m in modules if m.startswith('modules.{0}_'.format('module'))]
    return [getattr(importlib.import_module('modules.{0}_{1}'.format('module', entry_mod)), 'Module')().get_instance() for entry_mod in entry_mods]


class Realnet(Shell):

    def __init__(self):
        super().__init__('realnet')

    def get_commands(self):

        return [ItemCmd(), TypeCmd(), Device()] + Config.init().list_commands()

    def add_arguments(self, parser):
        pass

    def get_help(self):
        return 'realnet client'



from pynecone import ProtoShell, ProtoCmd


class TypeCmd(ProtoShell):

    class List(ProtoCmd):

        def __init__(self):
            super().__init__('list', 'list types')

    class Delete(ProtoCmd):

        def __init__(self):
            super().__init__('delete', 'delete a type')

    class Put(ProtoCmd):

        def __init__(self):
            super().__init__('put', 'put a type')

    class Get(ProtoCmd):

        def __init__(self):
            super().__init__('get', 'get a type')

    def __init__(self):
        super().__init__(   "type",
                            [TypeCmd.List(), TypeCmd.Delete(), TypeCmd.Get(), TypeCmd.Put() ],
                            "realnet types")




import pathlib
from setuptools import find_packages, setup

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

# This call to setup() does all the work
setup(
    name="realnet",
    version="0.0.10",
    description="Realnet command line interface",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/virtual-space/realnet",
    author="Marko Laban",
    author_email="marko.laban@l33tsystems.com",
    license="BSD",
    classifiers=[
        "License :: OSI Approved :: BSD License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
    ],
    packages=find_packages(exclude=("tests",)),
    include_package_data=True,
    install_requires=["pynecone==0.0.53",
                      "realnet-core",
                      "python-jsonstore==1.2.0",
                      "bleak",
                      "pybluez",
                      "tabulate"],
    entry_points={
        "console_scripts": [
            "realnet=realnet.__main__:main",
        ]
    },
)



from pynecone import ProtoShell, ProtoCmd
import logging
import sys

import bluetooth

import asyncio
from bleak import BleakScanner, BleakClient, BleakError
from bleak.uuids import uuidstr_to_str

GattCharacteristicsPropertiesEnum = {
    None: ("None", "The characteristic doesn’t have any properties that apply"),
    1: ("Broadcast", "The characteristic supports broadcasting"),
    2: ("Read", "The characteristic is readable"),
    4: ("WriteWithoutResponse", "The characteristic supports Write Without Response"),
    8: ("Write", "The characteristic is writable"),
    16: ("Notify", "The characteristic is notifiable"),
    32: ("Indicate", "The characteristic is indicatable"),
    64: ("AuthenticatedSignedWrites", "The characteristic supports signed writes"),
    128: ("ExtendedProperties", "The ExtendedProperties Descriptor is present"),
    256: ("ReliableWrites", "The characteristic supports reliable writes"),
    512: ("WritableAuxiliaries", "The characteristic has writable auxiliaries"),
}


class Device(ProtoShell):

    class Delete(ProtoCmd):

        def __init__(self):
            super().__init__('delete', 'delete a device')

    class Put(ProtoCmd):

        def __init__(self):
            super().__init__('put', 'put a device')

    class Get(ProtoCmd):

        def __init__(self):
            super().__init__('get', 'get a device')

    class Explore(ProtoCmd):

        class Discoverer(bluetooth.DeviceDiscoverer):
            def pre_inquiry(self):
                self.done = False

            def device_discovered(self, address, device_class, name):
                print("%s - %s - %s" % (address, name, device_class))

            def inquiry_complete(self):
                self.done = True


        def __init__(self):
            super().__init__('explore', 'explore a device')

        def add_arguments(self, parser):
            parser.add_argument('address', help="specifies the address of the device")

        def run(self, args):
            address = args.address
            loop = asyncio.get_event_loop()
            loop.run_until_complete(self.run_explore(address))

        async def run_explore(self, address):

            services = bluetooth.find_service(address=address)

            for service in services:
                print(service)

    class List(ProtoCmd):

        def __init__(self):
            super().__init__('list', 'list devices')

        def run(self, args):
            loop = asyncio.get_event_loop()
            loop.run_until_complete(self.run_discover())

        def run_discover(self):
            nearby_devices = bluetooth.discover_devices(lookup_class=True,lookup_names=True)

            for bdaddr in nearby_devices:
                print(bdaddr)


    def __init__(self):
        super().__init__(   "device",
                            [Device.Get(),
                             Device.Put(),
                             Device.List(),
                             Device.Explore(),
                             Device.Delete()],
                            "realnet devices")

